######################################
## Coder 4.x, RSTTool 3.x, Parser 3.x
#
#  File: Print.tcl
#  Author: Mick O'Donnell micko@wagsoft,com
#
#  Copyright: the author grants you a limited, nonexclusive, royalty-free 
#  right to reproduce and distribute this file; provided, however, that 
#  use of such software shall be subject to the terms and conditions of 
#  the Coder License Terms, a copy of which may be viewed at:
#  http://www.wagsoft.com/Coder/copyright.html
#  This notice must not be removed from the software, and in the event
#  that the software is divided, it should be attached to every part.

##########################
# Functionalities

# 1. myPrintCanvas: send whole canves to printer (Windows or Unix/Linux only)
# 2. myPrintWgts: send a set of canvas widgets to the printer
# 3. mySaveCanvas: save a canvas as either ps, pdx, wmf or emf
#    (only ps and pdx for Unix/Linux/Mac)
# 4. mySaveWgts: save a set of canvas widgets to a file
# 5. myCaptureCanvas: store the canvas in the clipboard for pasting (windows only)
# 6. myCaptureWgts: store the selected widgets into clipboard for pasting (windows only)

# Some functions work on Unix/Linux/Mac, but limited


#########################
# FILE SAVING
#########################


proc mySaveCanvas {cnv {file {}}} {

    # if no file, ask
    if { $file == {}} {
	set file [tk_getSaveFile -title "Save to file?"\
		-filetypes  { {PostScript {.eps .ps}} {PDX .pdx} } -initialfile "untitled.ps"]
    }

    if {$file == {}} {return}

    # find the filetype
    switch -- [string tolower [file extension $file]] {
	".ps" -
	".eps" {
	    if [file exists $file] {file delete $file}
	    $cnv postscript -file $file -pagex 0.i -pagey 11.i -pageanchor nw
	    tell-user "Postscript file saved"
	}
	
	".pdx" {
	    set wgts [$cnv find all]
	    mySaveWgts $cnv $wgts $file 
	}

	default {
	    tell-user "Sorry, cannot save canvas in format [file extension $file] yet."
	}
    }
}


proc mySaveWgts {cnv wgts file} {

    # find the filetype
    switch -- [string tolower [file extension $file]] {

	".ps" -
	".eps" {
	    # Copy the widgets to a new canvas, and then save the canvas
	    if [file exists $file] {file delete $file}
	    set cnv1 [copy-canvas $cnv $wgts]
	    $cnv1 postscript -file $file -pagex 0.i -pagey 11.i -pageanchor nw
	    destroy .tmp
	}

	".pdx" {
	    wgts-to-pdx $cnv $wgts $file
	}

	default {
	    tell-user "Sorry, cannot save canvas in format [file extension $file] yet."
	}
    }
}





#########################
# PRINT CANVAS
#########################

# 1. Print single page

proc myPrintCanvas {cnv} {

    set print_region [$cnv configure -scrollregion]
    set file /tmp/Tmp[pid]
    $cnv postscript -file $file \
	    -pagex 0.i -pagey 11.i -pageanchor nw
    busy-dialog .busdial "Printing $file on your default printer..."
    exec lpr $file 
    close-busy-dialog .busdial "Document printed!"
    file delete $file
}



proc myPrintWgts {cnv wgts} {
    global tcl_version
    set cnv1 [copy-canvas $cnv $wgts]
    myPrintCanvas $cnv1
}


#########################
# SUPPORT FUNCTIONS
#########################

proc copy-canvas {cnv wgts} {
    if [winfo exists .tmp] {destroy .tmp}
    toplevel .tmp
    canvas .tmp.canv -bg white
    pack .tmp.canv -fill both -expand t
    
    # 1. Capture the bbox of these widgets to find the xy offset
    # (We will set 0 to the top-lhs of the widgets)
    set bbox [eval [concat "$cnv bbox" $wgts]]
    set xoffset [lindex $bbox 0]
    set yoffset [lindex $bbox 1]

#BETTER - RETRIEVE ALL ARGS and substitute in new x/y
    foreach wgt $wgts {
	set coords1 [$cnv coords $wgt]
	# Correct the coords for offset
	set coords {}
	while { $coords1 != {}} {
	    set x [expr [pop coords1] - $xoffset]
	    set y [expr [pop coords1] - $yoffset]
	    lappend coords $x
	    lappend coords $y
	}

	set color [$cnv itemcget $wgt -fill]
	
	switch --  [$cnv type $wgt] {
	    text {
		
		set x [lindex $coords 0]
		set y [lindex $coords 1]
		set txt [$cnv itemcget $wgt -text]
		set attribs {}
		foreach attrib [$cnv itemconfigure $wgt] {
		    set var [lindex $attrib 0]
		    set val [lindex $attrib 3]
		    if { $val != {}} {
			switch -- $var {
			    "-font" {
				set attribs "$attribs [lindex $attrib 0] \{[lindex $attrib 4]\}"
			    }
			    default {
				set attribs "$attribs [lindex $attrib 0] [lindex $attrib 4]"
			    }
			}
		    }
		}
		eval [concat ".tmp.canv create text $x $y -text \{$txt\}" $attribs]
	    }
	    
	    line {
		set joinstyle [$cnv itemcget $wgt -joinstyle]
		set smooth [$cnv itemcget $wgt -smooth]
		if { $smooth == "bezier"} {set smooth t}
		eval [concat ".tmp.canv create line" $coords "-joinstyle $joinstyle"\
			"-smooth $smooth -fill $color"]
	    }
	    
	    arc { puts "Arc $wgt: $bbox"}
	    
	    rectangle {
		set outline [$cnv itemcget $wgt -outline]
		set attribs {}
		foreach attrib [$cnv itemconfigure $wgt] {
		    set val [lindex $attrib 3]
		    if { $val != {}} {
			set attribs "$attribs [lindex $attrib 0] [lindex $attrib 4]"
		    }
		}
		puts "RECT attribs: $attribs"
		eval [concat ".tmp.canv create rectangle" $coords $attribs]
	    }
	    
	    default {
		
		puts "Type: [$cnv type $wgt] not catered to"
	    }
	}
    }
    return .tmp.canv
}


proc addToCanvasMenu {menubar menu cnv} {
    MenuSeparator $menubar $menu
    MenuItem $menubar $menu "Print Diagram" "myPrintCanvas $cnv" 
    MenuItem $menubar $menu "Save Diagram As..." "save-canvas-as $cnv"  
}


